function [WeightsQP,WeightsEM1,WeightsEM2] = TaxyProWeights(WcountVec, StdVec, XprobMat, KernelMat, WindsRevComp)

% function [WeightsQP,WeightsEM] = TaxyWeights(WcountVec, StdVec, XprobMat, KernelMat, WindsRevComp)
%
% Description: calculate organisms-specific weights based on oligonucleotide
% counts of input (meta)genome in 'WcountVec' and reference signature
% probability estimates in XprobMat and corr. kernel matrix in 'KernelMat'.
% Performs quadratic programming and, in case of two output arguments,
% expectation-minimization, too.

if nargout>1
    emFlag=1;
    nIterEM = 500;
    nDiffEM1 = 1e-3;
    nDiffEM2 = 1e-4;
    DiffFlag1=0;
else
    emFlag=0;
end

nOrgs = size(XprobMat,2);

%% prepare metagenomic (test) signature   
if nargin > 4
    XcountVec = double(WcountVec) + double(WcountVec(WindsRevComp));
else
    XcountVec = double(WcountVec); 
end
Xvec = XcountVec / sum(XcountVec); % normalization
XvecStd = Xvec ./ StdVec; % standardization

%% quadratic program (QP)
lambda = 1e-8; % (very little) regularization for QP, might have a minor effect only for hexamers and large no of reference organisms!
Hmat = double(KernelMat + lambda*eye(nOrgs));
Qvec = double(((XvecStd./StdVec)' * XprobMat)');
Avec = ones(nOrgs,1);
b = 1;

[Ws, Dummy, Inds] = monqpCinfty(Hmat,Qvec,Avec,b, lambda,0);

WeightsQP = zeros(nOrgs,1);
WeightsQP(Inds) = Ws;

if emFlag
    Wvec = WeightsQP + 1e6; % initialization with large pseudocount!  
    Wvec = Wvec / sum(Wvec);
    
    for iter=1:nIterEM
        if ~mod(iter, 10); fprintf('.'); end
        % disp(iter);
        Pvec = Xvec ./ (XprobMat * Wvec);
        Pmat = XprobMat .* (Pvec * Wvec');
        Wvec = sum(Pmat)';
        Wvec = Wvec / sum(Wvec);
        %%%%>>>>Hoiner Hack%%%%
            if (iter > 1)
            DiffWVec=1/2*sum(abs(Wvecold-Wvec));
            %fprintf('%d\t%d\n',iter,DiffWVec);
                if (DiffWVec <= nDiffEM1 && DiffFlag1 == 0)
                DiffFlag1=1;
                WeightsEM1=Wvec;   
                end
                
                if (DiffWVec <= nDiffEM2)
                WeightsEM2 = Wvec;
                break;
                end
            end
        Wvecold=Wvec;
        
        %%%%<<<<Hoiner Hack%%%%
    end
fprintf('\n');

    if iter == nIterEM
        if ~exist('WeightsEM1')
        WeightsEM1=Wvec;
        end
    WeightsEM2 = Wvec;
    end

    
end
