function [WeightsEM,fdu] = TaxyProWeights2(ProfileMetVec, ProfileRefMat)
% function WeightsEM = TaxyProWeightsEM(ProfileMetVec, ProfileRefMat)
%
% Description: calculate organisms-specific weights based on protein domain
% counts of input (meta)genome in 'ProfileMetVec' and reference signature
% counts in ProfileRefMat 

nIterEM = 500;

%% prepare metagenomic (test) signature   

IndsNnz = find(sum(ProfileRefMat,2)>0 & ProfileMetVec>0);
XcountMat = single(ProfileRefMat);
[nDims,nOrgs] = size(XcountMat);
XprobFullMat = XcountMat ./ repmat(sum(XcountMat,1), nDims, 1); 
XprobMat = XprobFullMat(IndsNnz,:);

XcountVec = single(ProfileMetVec);
XvecFull = XcountVec / sum(XcountVec); % normalization
Xvec = XvecFull(IndsNnz);

Wvec = ones(nOrgs,1)/nOrgs; % start with equal distribution 
fduOld = 1; % maximum value

for iter=1:nIterEM
    Rvec = XprobMat * Wvec;
    fdu = sum(abs(Xvec - Rvec))/2;
    
    if (fduOld-fdu)/fduOld < 1e-4 
        break;
    else
        fduOld = fdu;
    end
    Pvec = Xvec ./ Rvec;
    Pmat = XprobMat .* (Pvec * Wvec');
    Wvec = sum(Pmat,1)';
    Wvec = Wvec / sum(Wvec);
end

WeightsEM = Wvec;
fdu = sum(abs(XvecFull - XprobFullMat*Wvec))/2;

%XprobMat = single(ProfileRefMat(IndsNnz,:))+eps('single');
end

